import { Component, EventEmitter, Input, Output, OnInit, forwardRef, ChangeDetectionStrategy } from '@angular/core';
import {
    NbpBaseComponent,
    NbpStyle,
    NbpDataSource
} from '@isp/xdce-widget';

import {ControlValueAccessor, FormControl, NG_VALUE_ACCESSOR, } from '@angular/forms';

//
// auto incremental index for noname combo instance
let comboIndex = 111;

@Component({
    selector: 'nbp-fid-combo',
    templateUrl: './nbp-fid-combo.component.html',
    styleUrls: ['./nbp-fid-combo.component.scss'],
    providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => NbpFidComboComponent),
            multi: true
        }
    ]
})
export class NbpFidComboComponent
    extends NbpBaseComponent 
    implements ControlValueAccessor {
    
    constructor() {
        super()
    }

    @Input() name: string;
    
    onChange: any = () => { };
    onTouch: any = () => {};
    
    compareFn(c1: any, c2: any): boolean {
        return c1 && c2 ? c1.value === c2.value : c1 === c2;
    }

    registerOnChange(fn: any): void {
        this.onChange = fn;
    }

    registerOnTouched(fn: any): void {
        this.onTouch = fn;
    }

    writeValue(input: string) {
        this.value = input;
    }

    @Input() nbpViewField : string = null;
    @Input() nbpKeyField : string = null;

    _emptyLabel: string = '';
    _emptyValue: string = null;
    _options: Array<any> = [];

    private _model: any;
    @Input()
    set value(value:  any) {
        this._model = value;
        if (this._model === undefined)
            this._model = null;

        // FUTURE handle a model value that is not present in options
        // as "empty" value
        // if (this._nbpShowEmptyValue && this._model !== null && !this.nbpViewField) {
        //     this._emptyValue = this._model;
        //     this._emptyLabel = this._model;
        // }

        this._selectOnModel();
    }

    @Output() ngModelChange = new EventEmitter<any>();
    get value(): any {
        return this._model;
    }

    private _nbpStyle:  NbpStyle = NbpStyle.DEFAULT;
    @Input()
    set nbpStyle(value:  NbpStyle) {
        this._nbpStyle = value;
    }
    get nbpStyle():  NbpStyle {
        return this._nbpStyle;
    }

    private _nbpShowEmptyValue: boolean = true;
    @Input()
    set nbpShowEmptyValue(value:  boolean) {
        this._nbpShowEmptyValue = value;
    }
    get nbpShowEmptyValue():  boolean {
        return this._nbpShowEmptyValue;
    }

    private _disabled: boolean = false;
    @Input()
    set disabled(value:  boolean) {
        this._disabled = value;
    }
    get disabled():  boolean {
        return this._disabled;
    }


    private _nbpDataSource:  NbpDataSource<any> = new NbpDataSource<any>();
    @Input()
    set nbpDataSource(value:  NbpDataSource<any>) {
        this._nbpDataSource = value;
        this._readOptions();
    }
    get nbpDataSource():  NbpDataSource<any> {
        return this._nbpDataSource;
    }

    ngOnInit(): void {
        if (!this.name) {
            this.name = "combo_" + (comboIndex++);
        }
        this._readOptions();
    }

    optStrValue(opt: any) {
        if (!opt)
            return "";
        if (typeof (opt) === "string")
            return opt;
        const v = this.nbpKeyField ? opt[this.nbpKeyField] : opt;
        if (v === null || v === undefined)
            return "";
        return "" + v;
    }

    optValue(opt: any) {
        if (!opt)
            return null;
        if (typeof (opt) === "string")
            return opt;
        const v = this.nbpKeyField ? opt[this.nbpKeyField] : opt;
        if (v === null || v === undefined)
            return null;
        return v;
    }

    
    optLabel(opt: any) {
        if (!opt)
            return '';
        if (typeof (opt) === "string")
            return opt;

        const l = this.nbpViewField ? opt[this.nbpViewField] : opt;
        if (l === null || l === undefined)
            return '';
        return "" + l;
    }

    private _readOptions() {
        this._options = this.nbpDataSource ? this.nbpDataSource.arrayData : [];
        if (!this._options)
            this._options = [];
    }

    
    _selectOnModel() {
        const arr = this._options;
        for (let a of arr) {
            a.selected = false;
        }
        const val = this.value;
        if (val === null || val === undefined)
            return;

        for (let a of arr) {
            const v = a.value;
            const sv = a.strValue;
            if (sv === val) {
                a.selected = true;
                break;
            }
        }
    }

    onValueChange(event: Event) {
        if (this._disabled)
            return;

        const selectElement = event.target as HTMLSelectElement;
        const val = selectElement.value || '';
        let sel = null;
        const arr = this._options;
        for (let a of arr) {
            const v = this.optValue(a);
            const sv = this.optStrValue(a);
            if (sv === val) {
                sel = v;
                break;
            }
        }

        this.value = sel;
        this.ngModelChange.emit(this.value);
        if (this.onChange)
            this.onChange(this.value);
    }
}